@@ -14,7 +14,7 @@ __all__ = [
     "get_process_group", "list_all_process_groups", "delete_process_group",
     "schedule_process_group", "create_process_group", "list_all_processors",
     "list_all_processor_types", "get_processor_type", 'create_processor',
-    'delete_processor', 'get_processor', 'schedule_processor',
+    'delete_processor', 'get_processor', 'schedule_processor', 'get_funnel',
     'update_processor', 'get_variable_registry', 'update_variable_registry',
     'purge_connection', 'purge_process_group', 'schedule_components',
     'get_bulletins', 'get_bulletin_board', 'list_invalid_processors',
@@ -23,8 +23,10 @@ __all__ = [
     'list_all_controllers', 'delete_controller', 'update_controller',
     'schedule_controller', 'get_controller', 'list_all_controller_types',
     'list_all_by_kind', 'list_all_input_ports', 'list_all_output_ports',
-    'list_all_funnels', 'list_all_remote_process_groups',
-    'get_remote_process_group', 'update_process_group'
+    'list_all_funnels', 'list_all_remote_process_groups', 'delete_funnel',
+    'get_remote_process_group', 'update_process_group', 'create_funnel',
+    'create_remote_process_group', 'delete_remote_process_group',
+    'set_remote_process_group_transmission', 'get_pg_parents_ids'
 ]
 
 log = logging.getLogger(__name__)
@@ -45,6 +47,8 @@ def recurse_flow(pg_id='root'):
     Returns information about a Process Group and all its Child Flows.
     Recurses the child flows by appending each process group with a
     'nipyapi_extended' parameter which contains the child process groups, etc.
+    Note: This previously used actual recursion which broke on large NiFi
+        environments, we now use a task/list update approach
 
     Args:
         pg_id (str): The Process Group UUID
@@ -54,20 +58,18 @@ def recurse_flow(pg_id='root'):
     """
     assert isinstance(pg_id, six.string_types), "pg_id should be a string"
 
-    def _walk_flow(node):
-        """This recursively extends the ProcessGroupEntity to contain the
-        ProcessGroupFlowEntity of each of it's child process groups.
-        So you can have the entire canvas in a single object.
-        """
-        if isinstance(node, nipyapi.nifi.ProcessGroupFlowEntity):
-            for pg in node.process_group_flow.flow.process_groups:
-                pg.__setattr__(
-                    'nipyapi_extended',
-                    recurse_flow(pg.id)
-                )
-            return node
-
-    return _walk_flow(get_flow(pg_id))
+    out = get_flow(pg_id)
+    tasks = [(x.id, x) for x in out.process_group_flow.flow.process_groups]
+    while tasks:
+        this_pg_id, this_parent_obj = tasks.pop()
+        this_flow = get_flow(this_pg_id)
+        this_parent_obj.__setattr__(
+            'nipyapi_extended',
+            this_flow
+        )
+        tasks += [(x.id, x) for x in
+                  this_flow.process_group_flow.flow.process_groups]
+    return out
 
 
 def get_flow(pg_id='root'):
@@ -85,10 +87,8 @@ def get_flow(pg_id='root'):
          (ProcessGroupFlowEntity): The Process Group object
     """
     assert isinstance(pg_id, six.string_types), "pg_id should be a string"
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.FlowApi().get_flow(pg_id)
-    except nipyapi.nifi.rest.ApiException as err:
-        raise ValueError(err.body)
 
 
 def get_process_group_status(pg_id='root', detail='names'):
@@ -121,7 +121,7 @@ def get_process_group_status(pg_id='root', detail='names'):
 def get_process_group(identifier, identifier_type='name'):
     """
     Filters the list of all process groups against a given identifier string
-    occuring in a given identifier_type field.
+    occurring in a given identifier_type field.
 
     Args:
         identifier (str): the string to filter the list for
@@ -134,7 +134,7 @@ def get_process_group(identifier, identifier_type='name'):
     """
     assert isinstance(identifier, six.string_types)
     assert identifier_type in ['name', 'id']
-    try:
+    with nipyapi.utils.rest_exceptions():
         if identifier_type == 'id':
             # assuming unique fetch of pg id
             # implementing separately to avoid recursing entire canvas
@@ -142,8 +142,6 @@ def get_process_group(identifier, identifier_type='name'):
         else:
             obj = list_all_process_groups()
             out = nipyapi.utils.filter_obj(obj, identifier, identifier_type)
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     return out
 
 
@@ -283,6 +281,7 @@ def list_all_processors(pg_id='root'):
     assert isinstance(pg_id, six.string_types), "pg_id should be a string"
 
     if nipyapi.utils.check_version('1.7.0') <= 0:
+        # Case where NiFi > 1.7.0
         targets = nipyapi.nifi.ProcessGroupsApi().get_processors(
             id=pg_id,
             include_descendant_groups=True
@@ -367,31 +366,11 @@ def delete_process_group(process_group, force=False, refresh=True):
     assert isinstance(process_group, nipyapi.nifi.ProcessGroupEntity)
     assert isinstance(force, bool)
     assert isinstance(refresh, bool)
+    pg_id = process_group.id
     if refresh or force:
-        target = nipyapi.nifi.ProcessGroupsApi().get_process_group(
-            process_group.id
-        )
+        target = nipyapi.nifi.ProcessGroupsApi().get_process_group(pg_id)
     else:
         target = process_group
-    if force:
-        # Stop, drop, and roll.
-        purge_process_group(target, stop=True)
-        # Remove inbound connections
-        for con in list_all_connections():
-            pg_id = process_group.id
-            if pg_id in [con.destination_group_id, con.source_group_id]:
-                delete_connection(con)
-        # Stop all Controller Services
-        for x in list_all_controllers(process_group.id):
-            delete_controller(x, True)
-        # Remove templates
-        for template in nipyapi.templates.list_all_templates(native=False):
-            if target.id == template.template.group_id:
-                nipyapi.templates.delete_template(template.id)
-        # have to refresh revision after changes
-        target = nipyapi.nifi.ProcessGroupsApi().get_process_group(
-            process_group.id
-        )
     try:
         return nipyapi.nifi.ProcessGroupsApi().remove_process_group(
             id=target.id,
@@ -399,6 +378,37 @@ def delete_process_group(process_group, force=False, refresh=True):
             client_id=target.revision.client_id
         )
     except nipyapi.nifi.rest.ApiException as e:
+        if force:
+            # Retrieve parent process group
+            parent_pg_id = nipyapi.canvas.get_process_group(pg_id, 'id')\
+                .component.parent_group_id
+            # Stop, drop, and roll.
+            purge_process_group(target, stop=True)
+            # Remove inbound connections
+            for con in list_all_connections(parent_pg_id):
+                if pg_id in [con.destination_group_id, con.source_group_id]:
+                    delete_connection(con)
+            # Stop all Controller Services ONLY inside the PG
+            controllers_list = list_all_controllers(pg_id)
+            removed_controllers_id = []
+            parent_pgs_id = get_pg_parents_ids(pg_id)
+            for x in controllers_list:
+                if x.component.id not in removed_controllers_id:
+                    if x.component.parent_group_id not in parent_pgs_id:
+                        delete_controller(x, True)
+                        removed_controllers_id.append(x.component.id)
+
+            # Remove templates
+            for template in nipyapi.templates.list_all_templates(native=False):
+                if target.id == template.template.group_id:
+                    nipyapi.templates.delete_template(template.id)
+            # have to refresh revision after changes
+            target = nipyapi.nifi.ProcessGroupsApi().get_process_group(pg_id)
+            return nipyapi.nifi.ProcessGroupsApi().remove_process_group(
+                id=target.id,
+                version=target.revision.version,
+                client_id=target.revision.client_id
+            )
         raise ValueError(e.body)
 
 
@@ -422,7 +432,7 @@ def create_process_group(parent_pg, new_pg_name, location, comment=''):
     assert isinstance(parent_pg, nipyapi.nifi.ProcessGroupEntity)
     assert isinstance(new_pg_name, six.string_types)
     assert isinstance(location, tuple)
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessGroupsApi().create_process_group(
             id=parent_pg.id,
             body=nipyapi.nifi.ProcessGroupEntity(
@@ -437,8 +447,6 @@ def create_process_group(parent_pg, new_pg_name, location, comment=''):
                 )
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise e
 
 
 def list_all_processor_types():
@@ -450,10 +458,8 @@ def list_all_processor_types():
          processors list
 
     """
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.FlowApi().get_processor_types()
-    except nipyapi.nifi.rest.ApiException as e:
-        raise e
 
 
 def get_processor_type(identifier, identifier_type='name'):
@@ -469,10 +475,8 @@ def get_processor_type(identifier, identifier_type='name'):
         list(Objects) for multiple matches
 
     """
-    try:
+    with nipyapi.utils.rest_exceptions():
         obj = list_all_processor_types().processor_types
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     if obj:
         return nipyapi.utils.filter_obj(obj, identifier, identifier_type)
     return obj
@@ -503,7 +507,7 @@ def create_processor(parent_pg, processor, location, name=None, config=None):
         target_config = nipyapi.nifi.ProcessorConfigDTO()
     else:
         target_config = config
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessGroupsApi().create_processor(
             id=parent_pg.id,
             body=nipyapi.nifi.ProcessorEntity(
@@ -519,8 +523,6 @@ def create_processor(parent_pg, processor, location, name=None, config=None):
                 )
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def get_processor(identifier, identifier_type='name'):
@@ -539,14 +541,12 @@ def get_processor(identifier, identifier_type='name'):
     """
     assert isinstance(identifier, six.string_types)
     assert identifier_type in ['name', 'id']
-    try:
+    with nipyapi.utils.rest_exceptions():
         if identifier_type == 'id':
             out = nipyapi.nifi.ProcessorsApi().get_processor(identifier)
         else:
             obj = list_all_processors()
             out = nipyapi.utils.filter_obj(obj, identifier, identifier_type)
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     return out
 
 
@@ -585,13 +585,11 @@ def delete_processor(processor, refresh=True, force=False):
         # refresh state before trying delete
         target = get_processor(processor.id, 'id')
         assert isinstance(target, nipyapi.nifi.ProcessorEntity)
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessorsApi().delete_processor(
             id=target.id,
             version=target.revision.version
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def schedule_components(pg_id, scheduled, components=None):
@@ -625,13 +623,11 @@ def schedule_components(pg_id, scheduled, components=None):
     )
     if components:
         body.components = {i.id: i.revision for i in components}
-    try:
+    with nipyapi.utils.rest_exceptions():
         result = nipyapi.nifi.FlowApi().schedule_components(
             id=pg_id,
             body=body
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     if result.state == target_state:
         return True
     return False
@@ -720,7 +716,7 @@ def update_process_group(pg, update):
 
         """
     assert isinstance(pg, nipyapi.nifi.ProcessGroupEntity)
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessGroupsApi().update_process_group(
             id=pg.id,
             body=nipyapi.nifi.ProcessGroupEntity(
@@ -732,8 +728,6 @@ def update_process_group(pg, update):
                 revision=pg.revision
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def update_processor(processor, update):
@@ -755,7 +749,7 @@ def update_processor(processor, update):
         raise ValueError(
             "update param is not an instance of nifi.ProcessorConfigDTO"
         )
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessorsApi().update_processor(
             id=processor.id,
             body=nipyapi.nifi.ProcessorEntity(
@@ -766,8 +760,6 @@ def update_processor(processor, update):
                 revision=processor.revision,
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def get_variable_registry(process_group, ancestors=True):
@@ -784,23 +776,22 @@ def get_variable_registry(process_group, ancestors=True):
         (VariableRegistryEntity): The Variable Registry
 
     """
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessGroupsApi().get_variable_registry(
             process_group.id,
             include_ancestor_groups=ancestors
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
-def update_variable_registry(process_group, update):
+def update_variable_registry(process_group, update, refresh=True):
     """
     Updates one or more key:value pairs in the variable registry
 
     Args:
         process_group (ProcessGroupEntity): The Process Group which has the
         Variable Registry to be updated
-        update (tuple[key, value]): The variables to write to the registry
+        update (list[tuple]): The variables to write to the registry
+        refresh (bool): Whether to refresh the object revision before updating
 
     Returns:
         (VariableRegistryEntity): The created or updated Variable Registry
@@ -825,7 +816,9 @@ def update_variable_registry(process_group, update):
             )
         ) for li in update
     ]
-    try:
+    with nipyapi.utils.rest_exceptions():
+        if refresh:
+            process_group = get_process_group(process_group.id, 'id')
         return nipyapi.nifi.ProcessGroupsApi().update_variable_registry(
             id=process_group.id,
             body=nipyapi.nifi.VariableRegistryEntity(
@@ -836,8 +829,6 @@ def update_variable_registry(process_group, update):
                 )
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def create_connection(source, target, relationships=None, name=None):
@@ -858,7 +849,7 @@ def create_connection(source, target, relationships=None, name=None):
     # determine source and destination strings by class supplied
     source_type = nipyapi.utils.infer_object_label_from_class(source)
     target_type = nipyapi.utils.infer_object_label_from_class(target)
-    if source_type not in ['OUTPUT_PORT', 'INPUT_PORT']:
+    if source_type not in ['OUTPUT_PORT', 'INPUT_PORT', 'FUNNEL']:
         source_rels = [x.name for x in source.component.relationships]
         if relationships:
             assert all(i in source_rels for i in relationships), \
@@ -878,7 +869,7 @@ def create_connection(source, target, relationships=None, name=None):
             parent_id = parent_pg.component.parent_group_id
     else:
         parent_id = source.component.parent_group_id
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ProcessGroupsApi().create_connection(
             id=parent_id,
             body=nipyapi.nifi.ConnectionEntity(
@@ -903,8 +894,6 @@ def create_connection(source, target, relationships=None, name=None):
                 )
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def delete_connection(connection, purge=False):
@@ -922,13 +911,11 @@ def delete_connection(connection, purge=False):
     assert isinstance(connection, nipyapi.nifi.ConnectionEntity)
     if purge:
         purge_connection(connection.id)
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.ConnectionsApi().delete_connection(
             id=connection.id,
             version=connection.revision.version
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def list_all_connections(pg_id='root', descendants=True):
@@ -997,10 +984,8 @@ def purge_connection(con_id):
             )
         return True
 
-    try:
+    with nipyapi.utils.rest_exceptions():
         drop_req = nipyapi.nifi.FlowfileQueuesApi().create_drop_request(con_id)
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     assert isinstance(drop_req, nipyapi.nifi.DropRequestEntity)
     return nipyapi.utils.wait_to_complete(_autumn_leaves, con_id, drop_req)
 
@@ -1043,10 +1028,8 @@ def get_bulletins():
         (ControllerBulletinsEntity): The native datatype containing a list
     of bulletins
     """
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.FlowApi().get_bulletins()
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def get_bulletin_board():
@@ -1056,10 +1039,8 @@ def get_bulletin_board():
     Returns:
         (BulletinBoardEntity): The native datatype BulletinBoard object
     """
-    try:
+    with nipyapi.utils.rest_exceptions():
         return nipyapi.nifi.FlowApi().get_bulletin_board()
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def create_controller(parent_pg, controller, name=None):
@@ -1080,7 +1061,7 @@ def create_controller(parent_pg, controller, name=None):
     assert isinstance(controller, nipyapi.nifi.DocumentedTypeDTO)
     assert isinstance(parent_pg, nipyapi.nifi.ProcessGroupEntity)
     assert name is None or isinstance(name, six.string_types)
-    try:
+    with nipyapi.utils.rest_exceptions():
         out = nipyapi.nifi.ProcessGroupsApi().create_controller_service(
             id=parent_pg.id,
             body=nipyapi.nifi.ControllerServiceEntity(
@@ -1098,8 +1079,6 @@ def create_controller(parent_pg, controller, name=None):
                     name=name
                 )
             )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     return out
 
 
@@ -1119,16 +1098,25 @@ def list_all_controllers(pg_id='root', descendants=True):
     assert isinstance(descendants, bool)
     handle = nipyapi.nifi.FlowApi()
     # Testing shows that descendant doesn't work on NiFi-1.1.2
-    if nipyapi.utils.check_version('1.1.2') < 1:
+    # Or 1.2.0, despite the descendants option being available
+    if nipyapi.utils.check_version('1.2.0') >= 0:
+        # Case where NiFi <= 1.2.0
         out = []
         if descendants:
             pgs = list_all_process_groups(pg_id)
         else:
             pgs = [get_process_group(pg_id, 'id')]
         for pg in pgs:
-            out += handle.get_controller_services_from_group(
+            new_conts = handle.get_controller_services_from_group(
                 pg.id).controller_services
+            # trim duplicates from inheritance
+            out += [
+                x for x in new_conts
+                if x.id not in [y.id for y in out]
+            ]
     else:
+        # Case where NiFi > 1.2.0
+        # duplicate trim already handled by server
         out = handle.get_controller_services_from_group(
             pg_id,
             include_descendant_groups=descendants
@@ -1142,7 +1130,7 @@ def delete_controller(controller, force=False):
 
     Args:
         controller (ControllerServiceEntity): Target Controller to delete
-        force (bool): True to Disable the Controller before deletion
+        force (bool): True to attempt Disable the Controller before deletion
 
     Returns:
         (ControllerServiceEntity)
@@ -1160,13 +1148,11 @@ def delete_controller(controller, force=False):
     if force:
         # Stop and refresh
         controller = schedule_controller(controller, False, True)
-    try:
+    with nipyapi.utils.rest_exceptions():
         result = handle.remove_controller_service(
             id=controller.id,
             version=controller.revision.version
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
     del_test = nipyapi.utils.wait_to_complete(
         _del_cont,
         controller.id,
@@ -1233,7 +1219,8 @@ def schedule_controller(controller, scheduled, refresh=False):
     if refresh:
         controller = nipyapi.canvas.get_controller(controller.id, 'id')
         assert isinstance(controller, nipyapi.nifi.ControllerServiceEntity)
-    if nipyapi.utils.check_version('1.1.2') < 1:
+    if nipyapi.utils.check_version('1.2.0') >= 0:
+        # Case where NiFi <= 1.2.0
         result = update_controller(
             controller=controller,
             update=nipyapi.nifi.ControllerServiceDTO(
@@ -1241,6 +1228,7 @@ def schedule_controller(controller, scheduled, refresh=False):
             )
         )
     else:
+        # Case where NiFi > 1.2.0
         result = handle.update_run_status(
             id=controller.id,
             body=nipyapi.nifi.ControllerServiceRunStatusEntity(
@@ -1372,6 +1360,94 @@ def get_remote_process_group(rpg_id, summary=False):
     return out
 
 
+def create_remote_process_group(target_uris, transport='RAW', pg_id='root',
+                                position=None):
+    """
+    Creates a new Remote Process Group with given parameters
+
+    Args:
+        target_uris (str): Comma separated list of target URIs
+        transport (str): optional, RAW or HTTP
+        pg_id (str): optional, UUID of parent Process Group for remote
+          process group
+        position (tuple): optional, tuple of location ints
+
+    Returns:
+        (RemoteProcessGroupEntity)
+    """
+    assert isinstance(target_uris, str)
+    assert transport in ['RAW', 'HTTP']
+    assert isinstance(pg_id, str)
+    pg_id = pg_id if not 'root' else get_root_pg_id()
+    position = position if position else (400, 400)
+    assert isinstance(position, tuple)
+    with nipyapi.utils.rest_exceptions():
+        return nipyapi.nifi.ProcessGroupsApi().create_remote_process_group(
+            id=pg_id,
+            body=nipyapi.nifi.RemoteProcessGroupEntity(
+                component=nipyapi.nifi.RemoteProcessGroupDTO(
+                    position=nipyapi.nifi.PositionDTO(
+                        x=float(position[0]),
+                        y=float(position[1])
+                    ),
+                    target_uris=target_uris,
+                    transport_protocol=transport
+                ),
+                revision=nipyapi.nifi.RevisionDTO(version=0),
+            )
+        )
+
+
+def delete_remote_process_group(rpg, refresh=True):
+    """
+    Deletes a given remote process group
+
+    Args:
+        rpg (RemoteProcessGroupEntity): Remote Process Group to remove
+        refresh (bool): Whether to refresh the object before action
+
+    Returns:
+        (RemoteProcessGroupEntity)
+    """
+    assert isinstance(rpg, nipyapi.nifi.RemoteProcessGroupEntity)
+    if refresh:
+        rpg = get_remote_process_group(rpg.id)
+    handle = nipyapi.nifi.RemoteProcessGroupsApi()
+    with nipyapi.utils.rest_exceptions():
+        return handle.remove_remote_process_group(
+            id=rpg.id,
+            version=rpg.revision.version
+        )
+
+
+def set_remote_process_group_transmission(rpg, enable=True, refresh=True):
+    """
+    Enable or Disable Transmission for an RPG
+
+    Args:
+        rpg (RemoteProcessGroupEntity): The ID of the remote process group
+          to modify
+        enable (bool): True to enable, False to disable
+        refresh (bool): Whether to refresh the object before action
+
+    Returns:
+
+    """
+    assert isinstance(rpg, nipyapi.nifi.RemoteProcessGroupEntity)
+    assert isinstance(enable, bool)
+    if refresh:
+        rpg = get_remote_process_group(rpg.id)
+    handle = nipyapi.nifi.RemoteProcessGroupsApi()
+    with nipyapi.utils.rest_exceptions():
+        return handle.update_remote_process_group_run_status(
+            id=rpg.id,
+            body=nipyapi.nifi.RemotePortRunStatusEntity(
+                state='TRANSMITTING' if enable else 'STOPPED',
+                revision=rpg.revision
+            )
+        )
+
+
 def create_port(pg_id, port_type, name, state, position=None):
     """
     Creates a new input or output port of given characteristics
@@ -1394,7 +1470,7 @@ def create_port(pg_id, port_type, name, state, position=None):
     assert isinstance(position, tuple)
     handle = nipyapi.nifi.ProcessGroupsApi()
     port_generator = getattr(handle, 'create_' + port_type.lower())
-    try:
+    with nipyapi.utils.rest_exceptions():
         return port_generator(
             id=pg_id,
             body=nipyapi.nifi.PortEntity(
@@ -1409,24 +1485,95 @@ def create_port(pg_id, port_type, name, state, position=None):
                 )
             )
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def delete_port(port):
     """Deletes a given port from the canvas if possible"""
     assert isinstance(port, nipyapi.nifi.PortEntity)
     if 'INPUT' in port.port_type:
-        try:
+        with nipyapi.utils.rest_exceptions():
             return nipyapi.nifi.InputPortsApi().remove_input_port(
                 id=port.id,
                 version=port.revision.version)
-        except nipyapi.nifi.rest.ApiException as e:
-            raise ValueError(e.body)
     if 'OUTPUT' in port.port_type:
-        try:
+        with nipyapi.utils.rest_exceptions():
             return nipyapi.nifi.OutputPortsApi().remove_output_port(
                 id=port.id,
                 version=port.revision.version)
-        except nipyapi.nifi.rest.ApiException as e:
-            raise ValueError(e.body)
+
+
+def get_funnel(funnel_id):
+    """Gets a given Funnel by ID"""
+    with nipyapi.utils.rest_exceptions():
+        return nipyapi.nifi.FunnelApi().get_funnel(funnel_id)
+
+
+def create_funnel(pg_id, position=None):
+    """
+    Creates a Funnel Object
+
+    Args:
+        pg_id (str): ID of the parent Process Group
+        position (tuple[int, int]): Position on canvas
+
+    Returns:
+        (FunnelEntity) Created Funnel
+    """
+    position = position if position else (400, 400)
+    assert isinstance(position, tuple)
+    with nipyapi.utils.rest_exceptions():
+        return nipyapi.nifi.ProcessGroupsApi().create_funnel(
+            id=pg_id,
+            body=nipyapi.nifi.FunnelEntity(
+                position=nipyapi.nifi.PositionDTO(
+                    x=float(position[0]),
+                    y=float(position[1])
+                ),
+                revision=nipyapi.nifi.RevisionDTO(version=0),
+                component=nipyapi.nifi.FunnelDTO(
+                    parent_group_id=pg_id
+                )
+            )
+        )
+
+
+def delete_funnel(funnel, refresh=True):
+    """
+    Deletes a Funnel Object
+
+    Args:
+        funnel (FunnelEntity): The Funnel to delete
+        refresh (bool): Whether to refresh the object state
+            before execution
+
+    Returns:
+        (FunnelEntity) Deleted FunnelEntity reference
+    """
+    assert isinstance(funnel, nipyapi.nifi.FunnelEntity)
+    with nipyapi.utils.rest_exceptions():
+        if refresh:
+            funnel = get_funnel(funnel.id)
+        return nipyapi.nifi.FunnelApi().remove_funnel(
+            id=funnel.id,
+            version=funnel.revision.version
+        )
+
+
+def get_pg_parents_ids(pg_id):
+    """
+    Retrieve the ids of the parent Process Groups.
+
+    Args:
+        pg_id (str): Process group id
+
+    Returns:
+        (list) List of ids of the input PG parents
+    """
+    parent_groups = []
+    while pg_id:
+        pg_id = nipyapi.canvas.get_process_group(pg_id, 'id') \
+            .component.parent_group_id
+        parent_groups.append(pg_id)
+    # Removing the None value
+    parent_groups.pop()
+    return parent_groups
