@@ -35,10 +35,8 @@ _valid_services = ['nifi', 'registry']
 def list_service_users(service='nifi'):
     """Lists all users of a given service, takes a service name as a string"""
     assert service in _valid_services
-    try:
+    with nipyapi.utils.rest_exceptions():
         out = getattr(nipyapi, service).TenantsApi().get_users()
-    except getattr(nipyapi, service).rest.ApiException as e:
-        raise ValueError(e.body)
     if service == 'nifi':
         return out.users
     return out
@@ -131,8 +129,7 @@ def create_service_user(identity, service='nifi', strict=True):
         )
     try:
         return getattr(nipyapi, service).TenantsApi().create_user(user_obj)
-    except (
-            nipyapi.nifi.rest.ApiException,
+    except (nipyapi.nifi.rest.ApiException,
             nipyapi.registry.rest.ApiException) as e:
         if 'already exists' in e.body and not strict:
             return get_service_user(identity, service=service)
@@ -148,7 +145,8 @@ def create_service_user_group(identity, service='nifi',
     Args:
         identity (str): Identity string for the user group
         service (str): 'nifi' or 'registry'
-        users (list): A list of UserEntities belonging to the group
+        users (list): A list of nifi.UserEntity or registry.User
+          belonging to the group
         strict (bool): Whether to throw an error on already exists
 
     Returns:
@@ -181,8 +179,7 @@ def create_service_user_group(identity, service='nifi',
         return getattr(nipyapi, service).TenantsApi().create_user_group(
             user_group_obj
         )
-    except (
-            nipyapi.nifi.rest.ApiException,
+    except (nipyapi.nifi.rest.ApiException,
             nipyapi.registry.rest.ApiException) as e:
         if 'already exists' in e.body:
             if not strict:
@@ -201,10 +198,8 @@ def list_service_user_groups(service='nifi'):
 
     """
     assert service in _valid_services
-    try:
+    with nipyapi.utils.rest_exceptions():
         out = getattr(nipyapi, service).TenantsApi().get_user_groups()
-    except getattr(nipyapi, service).rest.ApiException as e:
-        raise ValueError(e.body)
     if service == 'nifi':
         return out.user_groups
     return out
@@ -299,7 +294,6 @@ def service_login(service='nifi', username=None, password=None,
     log_args = locals()
     log_args['password'] = 'REDACTED'
     log.info("Called service_login with args %s", log_args)
-    # TODO: Tidy up logging and automate sensitive value redaction
     assert service in _valid_services
     assert username is None or isinstance(username, six.string_types)
     assert password is None or isinstance(password, six.string_types)
@@ -425,16 +419,15 @@ def get_service_access_status(service='nifi', bool_response=False):
             return False
         log.debug("- bool_response is False, raising Exception")
         raise e
-    except nipyapi.nifi.api_client.ApiException as e:
-        if 'only supported when running over HTTPS' in e.body:
-            return False
-        raise e
     except getattr(nipyapi, service).rest.ApiException as e:
-        if 'Authentication object was not found' in e.body:
+        expected_errors = [
+            'Authentication object was not found',
+            'only supported when running over HTTPS'
+        ]
+        if any(x in e.body for x in expected_errors):
             if bool_response:
                 return False
-            else:
-                raise e
+            raise e
 
 
 def add_user_to_access_policy(user, policy, service='nifi', refresh=True,
@@ -493,9 +486,8 @@ def add_user_to_access_policy(user, policy, service='nifi', refresh=True,
             policy_tgt.component.users.append({'id': user_id})
 
         return nipyapi.security.update_access_policy(policy_tgt, service)
-    else:
-        if strict and user_id in policy_user_ids:
-            raise ValueError("Strict is True and User ID already in Policy")
+    if strict and user_id in policy_user_ids:
+        raise ValueError("Strict is True and User ID already in Policy")
 
 
 def add_user_group_to_access_policy(user_group, policy, service='nifi',
@@ -575,13 +567,11 @@ def update_access_policy(policy, service='nifi'):
         nipyapi.registry.AccessPolicy if service == 'registry'
         else nipyapi.nifi.AccessPolicyEntity
     ), "Policy type {0} not vaid.".format(type(policy))
-    try:
+    with nipyapi.utils.rest_exceptions():
         return getattr(nipyapi, service).PoliciesApi().update_access_policy(
             id=policy.id if service == 'nifi' else policy.identifier,
             body=policy
         )
-    except nipyapi.nifi.rest.ApiException as e:
-        raise ValueError(e.body)
 
 
 def get_access_policy_for_resource(resource,
@@ -666,7 +656,9 @@ def create_access_policy(resource, action, r_id=None, service='nifi'):
     assert service in _valid_services
     if resource[0] != '/':
         r = '/' + resource
-    try:
+    else:
+        r = resource
+    with nipyapi.utils.rest_exceptions():
         if service == 'nifi':
             return nipyapi.nifi.PoliciesApi().create_access_policy(
                 body=nipyapi.nifi.AccessPolicyEntity(
@@ -684,9 +676,6 @@ def create_access_policy(resource, action, r_id=None, service='nifi'):
                 resource=r
             )
         )
-    except nipyapi.nifi.rest.ApiException as f:
-        log.info("Policy creation unsuccessful, raising error")
-        raise ValueError(f.body)
 
 
 def set_service_ssl_context(
@@ -749,11 +738,27 @@ def set_service_ssl_context(
     nipyapi.config.nifi_config.ssl_context = ssl_context
 
 
-def bootstrap_security_policies(service, user_identity=None):
-    assert service in _valid_services
+def bootstrap_security_policies(service, user_identity=None,
+                                group_identity=None):
+    """
+    Creates a default security context within NiFi or Nifi-Registry
+
+    Args:
+        service (str): 'nifi' or 'registry' to indicate which service
+        user_identity: a service user to establish in the security context
+        group_identity: a service group to establish in the security context
+
+    Returns:
+        None
+
+    """
+    assert service in _valid_services, "service not in %s" % _valid_services
+    valid_ident_obj = [nipyapi.nifi.UserEntity, nipyapi.registry.User]
+    if user_identity is not None:
+        assert user_identity in valid_ident_obj
     if 'nifi' in service:
         rpg_id = nipyapi.canvas.get_root_pg_id()
-        if user_identity is None:
+        if user_identity is None and group_identity is None:
             nifi_user_identity = nipyapi.security.get_service_user(
                 nipyapi.config.default_nifi_username,
                 service='nifi'
@@ -775,14 +780,27 @@ def bootstrap_security_policies(service, user_identity=None):
                 service='nifi',
                 auto_create=True
             )
-            nipyapi.security.add_user_to_access_policy(
-                user=nifi_user_identity,
-                policy=ap,
-                service='nifi',
-                strict=False
-            )
+            if nifi_user_identity is None:
+                # I should not rely upon a try/catch there
+                # but it's the simplest way (I just hope it won't
+                # break the server :-) )
+                try:
+                    nipyapi.security.add_user_group_to_access_policy(
+                        user_group=group_identity,
+                        policy=ap,
+                        service='nifi'
+                    )
+                except:  # noqa
+                    pass
+            else:
+                nipyapi.security.add_user_to_access_policy(
+                    user=nifi_user_identity,
+                    policy=ap,
+                    service='nifi',
+                    strict=False
+                )
     else:
-        if user_identity is None:
+        if user_identity is None and group_identity is None:
             reg_user_identity = nipyapi.security.get_service_user(
                 nipyapi.config.default_registry_username,
                 service='registry'
@@ -801,12 +819,19 @@ def bootstrap_security_policies(service, user_identity=None):
                 service='registry',
                 auto_create=True
             )
-            nipyapi.security.add_user_to_access_policy(
-                user=reg_user_identity,
-                policy=pol,
-                service='registry',
-                strict=False
-            )
+            if reg_user_identity is None:
+                nipyapi.security.add_user_group_to_access_policy(
+                    user_group=group_identity,
+                    policy=pol,
+                    service='registry'
+                )
+            else:
+                nipyapi.security.add_user_to_access_policy(
+                    user=reg_user_identity,
+                    policy=pol,
+                    service='registry',
+                    strict=False
+                )
         # Setup Proxy Access
         nifi_proxy = nipyapi.security.create_service_user(
             identity=nipyapi.config.default_proxy_user,
@@ -814,8 +839,9 @@ def bootstrap_security_policies(service, user_identity=None):
             strict=False
         )
         proxy_access_policies = [
+            ("read", "/proxy"),
             ("write", "/proxy"),
-            ("read", "/buckets")
+            ("delete", "/proxy"),
         ]
         for action, resource in proxy_access_policies:
             pol = nipyapi.security.get_access_policy_for_resource(
